/*
 * Copyright (c) 2009-2012 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * MEAS_TSYS01_Main.c: Console Application for Testing the TSYS01
 *
 * This application configures UART 16550 to baud rate 9600.
 * PS7 UART (Zynq) is not initialized by this application, since
 * bootrom/bsp configures it to baud rate 115200
 *
 * ------------------------------------------------
 * | UART TYPE   BAUD RATE                        |
 * ------------------------------------------------
 *   uartns550   9600
 *   uartlite    Configurable only in HW design
 *   ps7_uart    115200 (configured by bootrom/bsp)
 */

#include <stdio.h>
#include <unistd.h>
#include "platform.h"
#include "xparameters.h"
#include "tsys01.h"

#define		XPAR_AXI_IIC_JC_BASEADDR	XPAR_IIC_0_BASEADDR

void tsys01_main_menu(void);

int main()
{

    char key_input;
    char prom_read_flag=0;
    int i;
    tsys01_status stat;
    float temperature;

    //Initialize the UART
    init_platform();

    // Set the AXI address of the IIC core and
    // initialize the i2c address to 0x77
    tsys01_init(XPAR_AXI_IIC_JC_BASEADDR);

    // Display the main menu
    tsys01_main_menu();

    // Infinite loop
    while(1){

        // Get keyboard input
        read(1, (char*)&key_input, 1);

        if(key_input == '1'){               //If the '1' key is pressed

            // Display address selection menu
            printf("\n");
            printf("Select an address:\n");
            printf("  (0)   - CSB is set low  (Address=0x77)\n");
            printf("  (1)   - CSB is set high (Address=0x76)\n");

            // Get keyboard input ignoring keypresses that are not '0' or '1'
            read(1, (char*)&key_input, 1);
            while(key_input!='0' && key_input!='1'){
                read(1, (char*)&key_input, 1);
            }

            if(key_input == '0'){           // If the '0' key is pressed
                // Set i2c address to 0x77
                tsys01_set_address(tsys01_i2c_address_csb_0);
                printf("Set TSYS01 I2C Address to 0x77 (CSB low)\n");
            }else if(key_input == '1'){     // If the '1' key is pressed
                // Set i2c address to 0x76
                tsys01_set_address(tsys01_i2c_address_csb_1);
                printf("Set TSYS01 I2C Address to 0x76 (CSB high)\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys01_main_menu();

        }else if(key_input == '2'){         //If the '2' key is pressed

            // Send the reset command to the TSYS01
            printf("\n");
            printf("Resetting TSYS01...\n");
            stat = tsys01_reset();

            // Display the status returned from the reset operation
            printf("TSYS01 Reset Complete with status: ");
            if(stat==tsys01_status_ok)
                printf("Ok.\n");
            if(stat==tsys01_status_i2c_transfer_error)
                printf("Transfer Error.\n");

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys01_main_menu();

        }else if(key_input == '3'){         // If the '3' key is pressed

            // Read the PROM coefficients from the TSYS01
            printf("\n");
            printf("Reading PROM Coefficients...\n");
            stat = tsys01_read_prom();

            // Display status returned from read_prom operation
            // and display prom values if successful
            printf("Read PROM Complete with status: ");
            if(stat==tsys01_status_ok){
                prom_read_flag=1;
                printf("Ok.\n");
                printf("\n");
                printf("_________________________________________________\n");
                printf("|   PROM Addr   |Coeff (Base 10)|  Coeff (Hex)  |\n");
                printf("|---------------+---------------+---------------|\n");
                for(i=0;i<5;i++){
                    printf("|\t%d\t|     %5d\t|     0x%4X\t|\n",i,(unsigned int)tsys01_prom_coeffs[i],(unsigned int)tsys01_prom_coeffs[i]);
                }
            }else if(stat==tsys01_status_i2c_transfer_error){
                printf("Transfer Error.\n");
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys01_main_menu();

        }else if(key_input == '4'){         // If the '4' key is pressed

            if(prom_read_flag==0){          // PROM was not yet read--cannot read temperature yet
                printf("PROM Coefficients have not yet been read. Cannot complete temperature read.\n");
            }else{                          // PROM has been read--continue on to read temperature
                // Read one temperature value
                printf("\n");
                printf("Reading Temperature Value...\n");
                stat = tsys01_read_temperature(&temperature);

                // Display the status returned from the read_temperature
                // operation and display the temperature if successful
                printf("Temperature Read Complete with status: ");
                if(stat==tsys01_status_ok){
                    printf("Ok.\n");
                    printf("Temperature : %5.2f%cC\n",temperature,248);
                }else if(stat==tsys01_status_i2c_transfer_error){
                    printf("Transfer Error.\n");
                }
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys01_main_menu();

        }else if(key_input == '5'){         // If the '5' key is pressed

            if(prom_read_flag==0){          // PROM was not yet read--cannot read temperature yet
                printf("PROM Coefficients have not yet been read. Cannot complete temperature reads.\n");
            }else{                          // PROM has been read--continue on to read temperatures
                // Read 20 temperature values at ~2 per second
                printf("\n");
                printf("Reading 20 Temperature Values...\n");
                for(i=0;i<20;i++){
                    stat = tsys01_read_temperature(&temperature);
                    if(stat==tsys01_status_ok){
                        printf("%2d: %5.2f%cC\n",i+1,temperature,248);
                    }else{
                            printf("Transfer Error.\n");
                    }
                    usleep(490000);
                }
            }

            // Wait for another key press and then display the main menu again
            printf("\nPress any key to continue...\n");
            read(1, (char*)&key_input, 1);
            tsys01_main_menu();

        }else if(key_input == 27){    // If the 'ESC' key is pressed

            // Print done and exit.
            printf("Done.\n");
            break;

        }else{                      // If some other key is pressed

            // Redisplay the main menu
            tsys01_main_menu();

        }

    }

    return 0;

}

void tsys01_main_menu(void){

    //Clear the screen
    printf("\033[2J");

    //Display the main menu
    printf("*******************************************\n");
    printf("****      Measurement Specialties      ****\n");
    printf("*******************************************\n");

    printf("\n");
    printf("    TSYS01 - Digital Temperature Sensor    \n");
    printf("-------------------------------------------\n");

    printf("\n");
    printf("Select a task:\n");
    printf("  (1)   - Set TSYS01 I2C Address\n");
    printf("  (2)   - Reset TSYS01\n");
    printf("  (3)   - Read TSYS01 PROM Coefficients\n");
    printf("  (4)   - Read TSYS01 Temperature Once\n");
    printf("  (5)   - Read TSYS01 Temperature 20 Times\n");
    printf(" (ESC)  - Quit\n");
    printf("\n");

    return;
}
