/*
 * tsys01.c
 *
 *  Created on: Jul 30, 2014
 *      Author: JMBrinkhus
 */

#include "tsys01.h"
#include "xiic_l.h"
#include "sleep.h"
#include "math.h"
#include "xparameters.h"

// TSYS01 Global Variables
u32 			tsys01_axi_address;
tsys01_address 	tsys01_i2c_address;
u32 			tsys01_prom_coeffs[6];

/*********************************************************************
 *
 * Function:    tsys01_init
 *
 * Description:	Initialize the axi address of the i2c core and sets
 * 				a the default i2c address of the TSYS01 to 0x77
 *
 * Parameters:	u32 axi_address -
 * 					Address of Xilinx AXI IIC Peripheral
 *
 * Returns:		void
 *
 *********************************************************************/
void tsys01_init(u32 axi_address){

	tsys01_axi_address = axi_address;
	tsys01_i2c_address = tsys01_i2c_address_csb_0;

	return;

}

/*********************************************************************
 *
 * Function:    tsys01_set_address
 *
 * Description:	Sets the I2C address of the TSYS01
 *
 * Parameters:	tsys01_address addr -
 * 					Enumerated type passes value of either
 * 					address 0x77 or 0x76
 *
 * Returns:		void
 *
 *********************************************************************/
void tsys01_set_address(tsys01_address addr){
	tsys01_i2c_address = addr;
}

/*********************************************************************
 *
 * Function:    tsys01_reset
 *
 * Description:	Send I2C Reset command to TSYS01 and wait 100ms
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type tsys01_status with possible values
 * 				tsys01_status_ok or tsys01_status_i2c_transfer_error
 *
 *********************************************************************/
tsys01_status tsys01_reset(void){

	char tx_buf[1];
	int byte_count=0;

	tx_buf[0] = TSYS01_I2C_CMD_RESET;
	byte_count = XIic_Send(tsys01_axi_address, tsys01_i2c_address, (u8*)tx_buf, 1, XIIC_STOP);
	if(byte_count!=1){
		return tsys01_status_i2c_transfer_error;
	}
	//Device requires time to complete reset operation
	usleep(100000);

	return tsys01_status_ok;

}

/*********************************************************************
 *
 * Function:    tsys01_read_prom
 *
 * Description:	Read factory calibration values from PROM for later
 * 				use in converting ADC value to temperature
 *
 * Parameters:	void
 *
 * Returns:		Enumerated type tsys01_status with possible values
 * 				tsys01_status_ok or tsys01_status_i2c_transfer_error
 *
 *********************************************************************/
tsys01_status tsys01_read_prom(void){

	char tx_buf[1];
	char rx_buf[2];
	int i;
	int byte_count=0;

	//Repeat PROM read sequence 6 times to obtain calibration coefficients
	for(i=0; i<5; i++){
		tx_buf[0] = TSYS01_I2C_CMD_READ_PROM_BASE + 2*i;  //
		byte_count = XIic_Send(tsys01_axi_address, tsys01_i2c_address, (u8*)tx_buf, 1, XIIC_STOP);
		if(byte_count!=1){
			return tsys01_status_i2c_transfer_error;
		}
		byte_count = XIic_Recv(tsys01_axi_address, tsys01_i2c_address, (u8*)rx_buf, 2, XIIC_STOP);
		if(byte_count!=2){
			return tsys01_status_i2c_transfer_error;
		}

		tsys01_prom_coeffs[i] = 256*(u32)rx_buf[0] + (u32)rx_buf[1];
	}

	return tsys01_status_ok;

}


/*********************************************************************
 *
 * Function:    tsys01_read_temperature
 *
 * Description:	Start an ADC conversion, get the result, and compute
 * 				the measured temperature in degrees Celsius
 *
 * Parameters:	float* temperature -
 * 					Pointer to temperature variable.  This function
 * 					writes the temperature into this location.
 *
 * Returns:		Temperature in degrees Celsius
 *
 *********************************************************************/
tsys01_status tsys01_read_temperature(float* temperature){

	char tx_buf[1];
	char rx_buf[3];
	u16 adc16;
	int byte_count=0;

	// Send the i2c command to start an ADC conversion
	tx_buf[0] = TSYS01_I2C_CMD_START_ADC;
	byte_count = XIic_Send(tsys01_axi_address, tsys01_i2c_address, (u8*)tx_buf, 1, XIIC_STOP);
	// Verify the command sent successfully or return an error
	if(byte_count!=1){
		return tsys01_status_i2c_transfer_error;
	}

	// Wait 10ms for the ADC conversion to take place
    usleep(10000);

	// Send the i2c command to read the ADC values
	tx_buf[0] = TSYS01_I2C_CMD_READ_ADC;
	byte_count = XIic_Send(tsys01_axi_address, tsys01_i2c_address, (u8*)tx_buf, 1, XIIC_STOP);
	// Verify the command sent successfully or return an error
	if(byte_count!=1){
		return tsys01_status_i2c_transfer_error;
	}

	// Read the ADC values from the i2c bus
	byte_count = XIic_Recv(tsys01_axi_address, tsys01_i2c_address, (u8*)rx_buf, 3, XIIC_STOP);
	// Verify three bytes were read successfully or return an error
	if(byte_count!=3){
		return tsys01_status_i2c_transfer_error;
	}

	// Concatenate the received bytes into the 16 bit result
	adc16 = 256*rx_buf[0] + rx_buf[1];

	// Use PROM coefficients to compute temperature in degrees Celsius
	*temperature  = (-2.0) * tsys01_prom_coeffs[0] * pow(10, -21) * adc16 * adc16 * adc16 * adc16;
	*temperature += ( 4.0) * tsys01_prom_coeffs[1] * pow(10, -16) * adc16 * adc16 * adc16;
	*temperature += (-2.0) * tsys01_prom_coeffs[2] * pow(10, -11) * adc16 * adc16;
	*temperature += ( 1.0) * tsys01_prom_coeffs[3] * pow(10, -6)  * adc16;
	*temperature += (-1.5) * tsys01_prom_coeffs[4] * pow(10, -2);

	return tsys01_status_ok;
}
